/* ----
 * ---- file   : demo_checksum.c
 * ---- author : Bastian Spiegel <bs@tkscript.de>
 * ---- legal  : (c) 2013 by Bastian Spiegel.
 * ----          Distributed under terms of the MIT LICENSE (MIT).
 * ----
 * ---- Permission is hereby granted, free of charge, to any person obtaining a copy
 * ---- of this software and associated documentation files (the "Software"), to deal
 * ---- in the Software without restriction, including without limitation the rights
 * ---- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * ---- copies of the Software, and to permit persons to whom the Software is
 * ---- furnished to do so, subject to the following conditions:
 * ----
 * ---- The above copyright notice and this permission notice shall be included in
 * ---- all copies or substantial portions of the Software.
 * ----
 * ---- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * ---- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ---- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * ---- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * ---- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * ---- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * ---- THE SOFTWARE.
 * ----
 * ---- info   : This is part of the "c64_tools" package examples.
 * ----
 * ---- changed: 18Sep2013, 19Sep2013, 02Oct2013, 03Oct2013, 20Oct2013, 01Nov2013
 * ----
 * ----
 */

#define SYSCALLS_C defined
#include <libc64_dsp/include/inc_overlay.h>


#include "demo_checksum.h"


/* --------------------------------------------------------------------------- loc_rand() */
static sU32 seed = 0x44894489u;

static sU32 loc_rand_get(void) {
   seed *= 16807u;

   return seed;
}


/* --------------------------------------------------------------------------- loc_add() */
static sU32 loc_add(const sU32 *_addr, const sU32 _numBytes) {
   sUI ret = 0;
   sUI i;

   /* Invalidate DSP caches for the memory area */
   syscalls.cache_inv((void*)_addr, _numBytes, S_TRUE);

   for(i=0; i<(_numBytes >> 2); i++)
   {
      ret += _addr[i];
   }

   return ret;
}


/* --------------------------------------------------------------------------- loc_rand_write() */
static sU32 loc_rand_write(sU32 *_addr, const sU32 _numDWords) {
   sUI i;
   sU32 ret = 0;

   for(i=0; i<_numDWords; i++)
   {
      sU32 c = loc_rand_get();
      _addr[i] = c;

      ret = (ret ^ c);
   }

   syscalls.cache_wb(_addr, _numDWords * sizeof(sU32), S_TRUE /*wait*/);

   return ret;
}


/* --------------------------------------------------------------------------- loc_rand_read() */
static sU32 loc_rand_read(sU32 *_addr, const sU32 _numDWords) {
   sUI i;
   sU32 ret = 0;

   syscalls.cache_inv(_addr, _numDWords * sizeof(sU32), S_TRUE /*wait*/);

   for(i=0; i<_numDWords; i++)
   {
      sU32 c = _addr[i];

      ret = (ret ^ c);
   }

   syscalls.mlb_debug_usr(0, ret);
   ////syscalls->mlb_debug_usr(0, 0x87654321u);

   return ret;
}


/* --------------------------------------------------------------------------- exec */
static sU32 loc_exec(dsp_component_cmd_t _cmd,
                     sU32  _arg1, sU32  _arg2,
                     sU32 *_ret1, sU32 *_ret2
                     ) {
   sU32 ret;

   switch(_cmd)
   {
      default:
         /* Failed: illegal command id */
         ret = DEMO_CHECKSUM_ERR_ILLCMD;
         break;

      case DEMO_CHECKSUM_CMD_ADD:
         *_ret1 = loc_add((sU32*)_arg1,   /* phys_addr start */
                                 _arg2    /* num bytes */
                          );

         /* Succeeded */
         ret = DEMO_CHECKSUM_ERR_OK;
         break;

      case DEMO_CHECKSUM_CMD_RAND_WRITE:
         *_ret1 = loc_rand_write((sU32*)_arg1,   /* phys_addr start */
                                        _arg2    /* num dwords */
                                 );

         /* Succeeded */
         ret = DEMO_CHECKSUM_ERR_OK;
         break;

      case DEMO_CHECKSUM_CMD_RAND_READ:
         *_ret1 = loc_rand_read((sU32*)_arg1,   /* phys_addr start */
                                       _arg2    /* num dwords */
                                );

         /* Succeeded */
         ret = DEMO_CHECKSUM_ERR_OK;
         break;
   }

   return ret;
}


/* --------------------------------------------------------------------------- component_checksum_calc */
#pragma DATA_SECTION(component_demo_checksum,  ".sec_com");
dsp_component_t component_demo_checksum = {

   /* fxns: */
   {
      NULL,       /* init */
      &loc_exec,
      NULL,       /* exec fastcall RPC */
      NULL        /* exit */
   },

   COMPONENT_NAME_DEMO_CHECKSUM
};


DSP_COMPONENT_MAIN
